@@ -65,6 +65,7 @@
 #include "Logger.h"
 #include "LogFactory.h"
 #include "StringFormat.h"
+#include "RequestGroup.h"
 #include <cstring>
 
 namespace aria2 {
@@ -234,7 +235,7 @@ size_t DefaultBtInteractive::receiveMessages() {
   size_t msgcount = 0;
   for(int i = 0; i < 50; i++) {
     if(maxDownloadSpeedLimit > 0) {
-      TransferStat stat = peerStorage->calculateStat();
+      TransferStat stat = btContext->getOwnerRequestGroup()->calculateStat();
       if(maxDownloadSpeedLimit < stat.downloadSpeed) {
 	break;
       }
@@ -295,25 +296,32 @@ void DefaultBtInteractive::fillPiece(size_t maxMissingBlock) {
 
     if(peer->peerChoking()) {
       if(peer->isFastExtensionEnabled()) {
-
+	std::deque<size_t> excludedIndexes;
+	btRequestFactory->getTargetPieceIndexes(excludedIndexes);
 	while(numMissingBlock < maxMissingBlock) {
-	  PieceHandle piece = pieceStorage->getMissingFastPiece(peer);
+	  SharedHandle<Piece> piece =
+	    pieceStorage->getMissingFastPiece(peer, excludedIndexes);
 	  if(piece.isNull()) {
 	    break;
 	  } else {
 	    btRequestFactory->addTargetPiece(piece);
 	    numMissingBlock += piece->countMissingBlock();
+	    excludedIndexes.push_back(piece->getIndex());
 	  }
 	}
       }
     } else {
+      std::deque<size_t> excludedIndexes;
+      btRequestFactory->getTargetPieceIndexes(excludedIndexes);
       while(numMissingBlock < maxMissingBlock) {
-	PieceHandle piece = pieceStorage->getMissingPiece(peer);
+	SharedHandle<Piece> piece =
+	  pieceStorage->getMissingPiece(peer, excludedIndexes);
 	if(piece.isNull()) {
 	  break;
 	} else {
 	  btRequestFactory->addTargetPiece(piece);
 	  numMissingBlock += piece->countMissingBlock();
+	  excludedIndexes.push_back(piece->getIndex());
 	}
       }
     }
