@@ -46,6 +46,7 @@ class Request;
 class Segment;
 class Range;
 class Option;
+class CookieStorage;
 
 class HttpRequest {
 private:
@@ -58,8 +59,6 @@ private:
 
   uint64_t entityLength;
 
-  bool authEnabled;
-
   bool proxyEnabled;
 
   bool proxyAuthEnabled;
@@ -72,10 +71,11 @@ private:
 
   std::deque<std::string> _acceptTypes;
 
+  SharedHandle<CookieStorage> _cookieStorage;
+
   std::string getHostText(const std::string& host, uint16_t port) const;
 
   std::string getProxyAuthString() const;
-
 public:
   HttpRequest();
 
@@ -145,10 +145,8 @@ public:
   /**
    * Configures this object with option.
    * Following values are evaluated:
-   * PREF_HTTP_AUTH_ENABLED, PREF_HTTP_PROXY_ENABLED,
+   * PREF_HTTP_PROXY_ENABLED,
    * PREF_HTTP_PROXY_METHOD, PREF_HTTP_PROXY_AUTH_ENABLED,
-   * PREF_HTTP_USER, PREF_HTTP_PASSWD,
-   * PREF_HTTP_PROXY_USER, PREF_HTTP_PROXY_PASSWD
    * The evaluation results are stored in instance variables.
    */
   void configure(const Option* option);
@@ -163,11 +161,6 @@ public:
     this->proxyAuthEnabled = proxyAuthEnabled;
   }
 
-  void setAuthEnabled(bool authEnabled)
-  {
-    this->authEnabled = authEnabled;
-  }
-
   void enableContentEncoding();
 
   void disableContentEncoding();
@@ -187,6 +180,10 @@ public:
   {
     _acceptTypes.insert(_acceptTypes.end(), first, last);
   }
+
+  void setCookieStorage(const SharedHandle<CookieStorage>& cookieStorage);
+
+  SharedHandle<CookieStorage> getCookieStorage() const;
 };
 
 typedef SharedHandle<HttpRequest> HttpRequestHandle;
